


const Product = require("../models/productsModel");
const { v2: cloudinary } = require("cloudinary");
const { createShopifyProduct, fetchShopifyProductDetails } = require('../shopify/shopifyProducts');



// Main function to handle product creation and saving in the database
const productCreate = async (req, res) => {
    try {
      // Check if the image files are included in the request
      const files = req.files ? (Array.isArray(req.files.images) ? req.files.images : [req.files.images]) : [];
  
      // Upload all images to Cloudinary if present
      const uploadPromises = files.map((file) =>
        cloudinary.uploader.upload(file.tempFilePath, {
          folder: 'products',
          use_filename: true,
        })
      );
      const uploadResults = await Promise.all(uploadPromises);
      const mediaUrls = uploadResults.map((result) => result.secure_url);
  
      // Prepare product data for Shopify
      const shopifyProductInput = {
        title: req.body.name || 'Unnamed Product',
        descriptionHtml: req.body.description || '',
        vendor: req.body.vendor || '',
        productType: req.body.productType || '',
        tags: Array.isArray(req.body.tags) ? req.body.tags.join(', ') : req.body.tags || '',
        options: req.body.options.map((option) => ({
          name: option.name,
          values: option.values,
        })), // Correctly defined options like Size
        variants: req.body.variants.map((variant) => ({
          option1: variant.optionName, // Assuming optionName is for option1 (e.g., Size)
          price: variant.price || '0.00', // Shopify expects price as string
          inventory_quantity: variant.available || 0, // Inventory for this variant
        })),
        media: mediaUrls.map((url) => ({
          alt: 'Product Image', // Replace with any appropriate alt text
          mediaContentType: 'IMAGE',
          src: url,
        })), // Media instead of images
      };
  
      // Step 1: Create the product in Shopify
      const shopifyResponse = await createShopifyProduct(shopifyProductInput);
  
      if (shopifyResponse.errors || shopifyResponse.data.productCreate.userErrors.length > 0) {
        return res.status(500).json({
          message: 'Failed to create product in Shopify',
          errors: shopifyResponse.errors || shopifyResponse.data.productCreate.userErrors,
        });
      }
  
      const shopifyProductId = shopifyResponse.data.productCreate.product.id;
  
      // Step 2: Fetch the full product details from Shopify
      const productDetailsResponse = await fetchShopifyProductDetails(shopifyProductId);
      const shopifyProduct = productDetailsResponse.data.product;
  
      // Step 3: Now use the Shopify response to create the product in MongoDB
      const newProduct = new Product({
        admin_graphql_api_id: shopifyProductId,
        title: shopifyProduct.title,
        body_html: shopifyProduct.descriptionHtml,
        vendor: shopifyProduct.vendor,
        product_type: shopifyProduct.productType,
        totalInventory: shopifyProduct.totalInventory,
        handle: shopifyProduct.handle,
        published_scope: shopifyProduct.publishedScope,
        tags: Array.isArray(shopifyProduct.tags) ? shopifyProduct.tags.join(', ') : shopifyProduct.tags || '',
        variants: shopifyProduct.variants.nodes.map((variant) => ({
          title: variant.title,
          price: parseFloat(variant.price),
          inventory_quantity: variant.inventoryQuantity,
          admin_graphql_api_id: variant.id,
        })),
        images: mediaUrls.length ? mediaUrls : shopifyProduct.images.nodes.map((image) => image.src),
      });
  
      // Save the product in the local database
      const savedProduct = await newProduct.save();
  
      // Send success response
      res.status(201).json({
        message: 'Product created successfully in Shopify and local DB',
        product: savedProduct,
        shopifyProductId: shopifyProductId,
      });
    } catch (error) {
      console.error('Error creating product:', error);
  
      // Handle Mongoose validation errors
      if (error.name === 'ValidationError') {
        const errors = Object.values(error.errors).map((err) => err.message);
        return res.status(400).json({ message: 'Product validation failed', errors });
      }
  
      // General error handling
      res.status(500).json({ message: 'Failed to create product', error: error.message });
    }
  };




// Get all products with pagination
const getAllProducts = async (req, res) => {
  try {
    const page = parseInt(req.query.page) || 1; // Get the page number from query params, default to 1
    const limit = parseInt(req.query.limit) || 10; // Get the number of items per page from query params, default to 10
    const skip = (page - 1) * limit; // Calculate how many products to skip

    // Fetch the total number of products
    const total = await Product.countDocuments();

    // Fetch the products with pagination
    const products = await Product.find().skip(skip).limit(limit);

    // Send the products, message, total number of products, and current page as a response
    res.status(200).json({
      message: "Products fetched successfully",
      total: total, // Total number of products
      currentPage: page, // Current page number
      totalPages: Math.ceil(total / limit), // Total number of pages
      products: products, // Products for the current page
    });
  } catch (error) {
    console.error("Error fetching products:", error);
    res
      .status(500)
      .json({ message: "Failed to fetch products", error: error.message });
  }
};

// Delete a product by ID
const deleteProduct = async (req, res) => {
  try {
    // console.log(req,"reqreq")
    const productId = req.params._id;
    console.log(productId);

    const deletedProduct = await Product.findByIdAndDelete(productId);

    if (!deletedProduct) {
      return res.status(200).json({ message: "Product not found" });
    }

    return res.status(404).json({
      message: "Product deleted successfully",
      product: deleteProduct,
    });
  } catch (error) {
    console.error("Error deleting product:", error);
    res
      .status(500)
      .json({ message: "Failed to delete product", error: error.message });
  }
};

// Update product by ID
const updateProduct = async (req, res) => {
  try {
    const productId = req.params._id;
    const updateData = req.body;

    const updatedProduct = await Product.findByIdAndUpdate(
      productId,
      updateData,
      { new: true, runValidators: true }
    );
    if (!updatedProduct) {
      return res.status(200).json({ message: "Product not found" });
    }

    res.status(200).json({
      message: "Product updated successfully",
      product: updatedProduct,
    });
  } catch (error) {
    console.error("Error updating product:", error);

    if (error.name === "ValidationError") {
      const errors = Object.values(error.errors).map((err) => err.message);
      return res
        .status(400)
        .json({ message: "Product validation failed", errors });
    }

    res
      .status(500)
      .json({ message: "Failed to update product", error: error.message });
  }
};

module.exports = {
    productCreate,
  getAllProducts,
  deleteProduct,
  updateProduct,
};
