const axios = require('axios');
require('dotenv').config(); // To load environment variables from .env


// Function to create a product in Shopify
const createShopifyProduct = async (productInput) => {
    try {
      // Define headers including Shopify access token from .env
      const headers = {
        'Content-Type': 'application/json',
        'X-Shopify-Access-Token': process.env.SHOPIFY_ACCESS_TOKEN,
      };
  
      // GraphQL mutation query to create a product
      const graphqlQuery = {
        query: `
          mutation createProduct($input: ProductInput!) {
            productCreate(input: $input) {
              product {
                id
                title
                descriptionHtml
                vendor
                productType
                totalInventory
                options {
                  name
                  values
                }
                variants(first: 10) {
                  nodes {
                    id
                    title
                    price
                    inventoryQuantity
                  }
                }
              }
              userErrors {
                message
                field
              }
            }
          }
        `,
        variables: {
          input: productInput,
        },
      };
  
      // Axios request to Shopify's GraphQL API
      const response = await axios.post(
        `https://${process.env.SHOPIFY_STORE_URL}/admin/api/2024-07/graphql.json`,
        graphqlQuery,
        { headers }
      );
  
      // Return the product data or errors if any
      return response.data;
    } catch (error) {
      console.error('Error creating Shopify product:', error.response ? error.response.data : error.message);
      throw new Error('Failed to create product in Shopify');
    }
  };




















// Function to fetch product details from Shopify by product ID
const fetchShopifyProductDetails = async (shopifyProductId) => {
  try {
    // Define headers including Shopify access token from .env
    const headers = {
      'Content-Type': 'application/json',
      'X-Shopify-Access-Token': process.env.SHOPIFY_ACCESS_TOKEN,
    };

    // GraphQL query to fetch product details by ID
    const graphqlQuery = {
      query: `
        query getProductDetails($id: ID!) {
          product(id: $id) {
            id
            title
            descriptionHtml
            vendor
            productType
            totalInventory
            variants(first: 10) {
              nodes {
                id
                title
                price
                inventoryQuantity
              }
            }
            images(first: 10) {
              nodes {
                altText
                src
              }
            }
            tags
            handle
          }
        }
      `,
      variables: {
        id: shopifyProductId, // Pass shopifyProductId directly
      },
    };

    // Axios request to Shopify's GraphQL API
    const response = await axios.post(
      `https://${process.env.SHOPIFY_STORE_URL}/admin/api/2024-07/graphql.json`,
      graphqlQuery,
      { headers }
    );

    // Return the product details
    return response.data;
  } catch (error) {
    console.error('Error fetching Shopify product details:', error.response ? error.response.data : error.message);
    throw new Error('Failed to fetch product details from Shopify');
  }
};

  
module.exports = { createShopifyProduct, fetchShopifyProductDetails };
